module oscil3 (adc_clk1, adc_clk2,                 //  
               adc1_data,                          //   ADC
               ram_data_debug,                     //      sram
               epm_data, epm_wr, epm_rs, epm_cs,   //    
               ram_adr, ram_data, ram_wr, ram_oe); //    sram
               
output reg [15:0] ram_data_debug;
//output reg [15:0] data_debug;

input wire        adc_clk1;   //   
input wire        adc_clk2;   //    

//   SRAM
output wire [17:0] ram_adr;
assign             ram_adr=ram_dir ? wr_ram_adr : rd_ram_adr;

inout wire [15:0] ram_data;
assign            ram_data=ram_dir ? wr_ram_data : 16'bZZZZZZZZZZZZZZZZ;

output reg        ram_wr;      //    SRAM
output reg        ram_oe;      //     SRAM

//   ADC
input wire [7:0]  adc1_data;   //  ADC 

//    
inout wire [7:0]  epm_data;    //  \  
assign            epm_data = epm_wr ? out_epm_data : 8'bZZZZZZZZ;
reg        [7:0]  out_epm_data; 

input wire        epm_wr;      //   \ 

input wire        epm_rs;      //  \

input wire        epm_cs;      //   

initial
begin
   ram_dir=1;    //    SRAM -  

   sync_start=1; //     
   sync_simple_type=0; //   -  
   
   capt_start=0;  //      
   one_capt=0;    //   
   capt_stop=0;   //      
   
   ram_wr=1;
   ram_oe=1;

   prediv_value[15:0]=1;
   prediv16<=prediv_value;
end

//----------------------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------------------
reg        [15:0] prediv_value; //   
reg        [15:0] prediv16;     //    
reg               one_capt;     //   
reg               capt_start;   //   -     
/*  :
capt_start=1 & capt_stop=0       (one_capt)
*/
always @ (negedge adc_clk1)  //  
begin  
  prediv16<=prediv16-1; //     

  if (prediv16==1)      //    (  1,    0)
     begin
        prediv16<=prediv_value;      //    
        
        one_capt<=~capt_stop & capt_start ;  //     
     end 
        else one_capt<=0; //   
end



//----------------------------------------------------------------------------------------------------------------
//  :   wr
//----------------------------------------------------------------------------------------------------------------
reg               capt_step;         //  
reg               out_data;
/*  
one_capt==1     (  capt_step  ram_wr
*/
always @ (negedge adc_clk1) // 
begin
  case (capt_step)
    0: begin
         ram_wr<=1;
         if (one_capt==1) 
           begin
             capt_step<=1;
             out_data<=1;
           end
       end
    1: begin
          ram_wr<=0;
          capt_step<=0;
          out_data<=0;
       end
  endcase
end



//----------------------------------------------------------------------------------------------------------------
//  :      
//----------------------------------------------------------------------------------------------------------------
// :
reg        [15:0] wr_ram_data;       //      SRAM
reg        [7:0]  adc_temp;          //      ADC
reg        [17:0] wr_ram_adr;        //  ram  
/*  :
out_data==1  ,  !
     :
capt_step -   
capt_start -      ( =0 -      SRAM)
*/
always @ (negedge adc_clk2) // 
begin
      case (capt_step)
    1: if (out_data==1)
          begin   
            wr_ram_data[15:8]<=adc1_data[7:0]; //  
            wr_ram_data[7:0]<=adc_temp[7:0];

            wr_ram_adr<=wr_ram_adr+1; //adr_inc_needed; //         

            ram_data_debug[15:8]<=adc1_data[7:0]; //    sram
            ram_data_debug[7:0]<=adc_temp[7:0];
          end else
              if (capt_start==0) wr_ram_adr<=0;   //     
    0: begin
          adc_temp<=adc1_data;
       end
  endcase
end


//----------------------------------------------------------------------------------------------------------------
//      
//----------------------------------------------------------------------------------------------------------------
// 
reg         capt_stop; //      "1"
/*  
 wr_ram_adr -   
*/
always @ (negedge adc_clk1) //     
begin //      -   !
  if (capt_start==1)
    begin
      if (wr_ram_adr==18'b111111111111111110) capt_stop<=1;
    end else capt_stop<=0;
end


//----------------------------------------------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------------------------------------------
reg [2:0]    sync_simple_type;  //   
                             // 0 -  
                             // 1 -    
                             // 2 -    
                             // 3 -    
                             // 4 -    
reg [7:0]      sync_value;
reg [7:0]      sync_temp;    //   
reg            sync_start;   //     
always @ (negedge adc_clk1) // 
begin
   if (sync_start)
     case (sync_simple_type)
       0: begin //  
            capt_start<=1;
          end
       1: begin //    
            if (sync_temp<sync_value) capt_start<=1;
          end
       2: begin //    
            if (sync_temp>sync_value) capt_start<=1;   
          end
       3: begin //    
            if (sync_temp<adc1_data) capt_start<=1;   
          end
       4: begin //    
            if (sync_temp>adc1_data) capt_start<=1;   
          end
     endcase
   else capt_start<=0;
   
   sync_temp<=adc1_data;   
end

//----------------------------------------------------------------------------------------------------------------
//   
//----------------------------------------------------------------------------------------------------------------
reg  [7:0]  epm_reg;           //     
reg         ram_dir;      //   SRAM (1 , 0 )
reg  [17:0] rd_ram_adr;        //  ram  
reg  [7:0]  rd_ram_step;       //     

always @ (negedge adc_clk2)
begin
   if (epm_cs==1)
      case (epm_wr)
        0 : begin //   
              case (epm_rs)
                0 : begin //   
                      epm_reg[7:0]<=epm_data[7:0];
                    end
                1 : begin //    
                      case (epm_reg)
                        0: begin //  
                             if (epm_data[0]==1) sync_start<=1; //     
                           end
                        1: begin //   ( )
                             rd_ram_adr[7:0]<=epm_data[7:0];
                             ram_dir<=0;
                             ram_oe<=0;
                             sync_start<=0;
                           end
                        2: begin //   ( )
                             rd_ram_adr[15:8]<=epm_data[7:0];
                             ram_dir<=0;
                             ram_oe<=0;
                             sync_start<=0;
                           end
                        3: begin //   ( )
                             rd_ram_adr[17:16]<=epm_data[1:0];
                             ram_dir<=0;
                             ram_oe<=0;
                             sync_start<=0;
                           end
                        4: begin //    
                             rd_ram_step[7:0]<=epm_data[7:0];
                             ram_dir<=0;
                             ram_oe<=0;    
                             sync_start<=0;                         
                           end
                        5: begin //  :  
                             prediv_value[7:0]<=epm_data[7:0];
                             ram_dir<=1;
                             ram_oe<=0;
                             sync_start<=0;
                           end
                        6: begin //  :  
                             prediv_value[15:8]<=epm_data[7:0];
                             ram_dir<=1;
                             ram_oe<=1;
                             sync_start<=0;
                           end
                        7: begin //  
                             sync_simple_type[2:0]<=epm_data[2:0];
                             ram_dir<=1;
                             ram_oe<=1;
                             sync_start<=0;
                           end
                        8: begin //  
                             sync_value[7:0]<=epm_data[7:0];
                             ram_dir<=1;
                             ram_oe<=1;
                             sync_start<=0;
                           end                          
                      endcase
                    end
              endcase
            end
        1 : begin //   
               case (epm_reg)
                  0: begin //  
                     end
                  1: begin // SRAM ( )
                       out_epm_data[7:0]<=ram_data[7:0];
                     end
                  2: begin // SRAM ( )
                       out_epm_data[7:0]<=ram_data[15:8];
                     end
                  3: begin // 
                     end
                  4: begin // 
                     end
                  5: begin // 
                     end
                  6: begin // 
                     end
                  7: begin // 
                     end
                  8: begin // 
                     end                          
               endcase            
            end
      endcase
end
endmodule
